/**
 * Fichier : PC_Epd_154b_low_memory pour MSP430FR4133
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 23 fvrier 2018
 * (c) Pascal Chour 
 * Bas sur epd1in54b
 * Utilisable librement. Merci de prciser la source.
 * 
 *-------------------------------------------------------------------------- 
 * Cette bibliothque permet de grer un afficheur e-paper 1.54 pouce type B
 * (pas de rafraichissement partiel) sur un microcontrleur ne disposant que de
 * peu de RAM (typiquement, 2ko comme pour certains Arduino)
 *
 * Cette version de la bibliothque est conue pour tre utilise sur une carte
 * MSP430F4133 et utilise SPI B.
 *
 * La bibliothqe permet :
 * - d'initialiser l'afficheur (cf. Waveshare)
 * - d'effacer l'cran
 * - de remplir l'cran
 * - d'afficher des lignes horizontale 
 * - d'afficher des textes
 * Le tout en utilisant trs peu de RAM (moins de 300 octets de mmoire temporaire).
 *--------------------------------------------------------------------------
 * This library can managed a 1.54 inch e-paper display type B (no partial refresh) with
 * a low memory microcontroller (typically : some Arduino with 2kb RAM).
 * The functionalities of the library are :
 * - display management (init..., see Waveshare)
 * - clear the display
 * - fill the display
 * - draw horizontal line
 * - write text
 * The RAM used is lower than 300 bytes (temporary RAM).
 *--------------------------------------------------------------------------
 *
 * Some functions came from
 *  @filename   :   epd1in54b.cpp
 *  @brief      :   Implements for e-paper library
 *  @author     :   Yehui from Waveshare
 *
 *  Copyright (C) Waveshare     August 10 2017
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documnetation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to  whom the Software is
 * furished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "PC_Epd_154_low_memory.h"
#include "common.h"
#include "driverlib/MSP430FR2xx_4xx/driverlib.h"
#include "utilitaires.h"

#include "Font20.h"
#include "Font15.h"
#include "Font4line.h"

//#define AFFICHAGE_RAPIDE

static uint16_t CurrentByteBlack = 0;	    // Position courante sur banc BLACK
static uint16_t CurrentByteRed = 0;	        // Position courante sur banc RED
static unsigned char CurrentBank = BLACK; 	// banc courant



sFONT Font20 = {
  Font20_Table,
  17,   // Width
  20,   // Height
  0x20, // Base, premier caractre = Espace, 2me caractre = 0x21
};

sFONT Font15 = {
  Font15_Table,
  17,   // Width
  15,   // Height
  0x20, // Base, premier caractre = Espace, 2me caractre = 0x21
};

sFONT Font4line = {
  Font4line_Table,
  17,   // Width
  4,    // Height
  0x20, // Base, premier caractre = Espace, 2me caractre = 0x21
};

const unsigned char lut_vcom0[] =
{
    0x0E, 0x14, 0x01, 0x0A, 0x06, 0x04, 0x0A, 0x0A,
    0x0F, 0x03, 0x03, 0x0C, 0x06, 0x0A, 0x00
};

const unsigned char lut_w[] =
{
    0x0E, 0x14, 0x01, 0x0A, 0x46, 0x04, 0x8A, 0x4A,
    0x0F, 0x83, 0x43, 0x0C, 0x86, 0x0A, 0x04
};

const unsigned char lut_b[] =
{
    0x0E, 0x14, 0x01, 0x8A, 0x06, 0x04, 0x8A, 0x4A,
    0x0F, 0x83, 0x43, 0x0C, 0x06, 0x4A, 0x04
};

const unsigned char lut_g1[] =
{
    0x8E, 0x94, 0x01, 0x8A, 0x06, 0x04, 0x8A, 0x4A,
    0x0F, 0x83, 0x43, 0x0C, 0x06, 0x0A, 0x04
};

const unsigned char lut_g2[] =
{
    0x8E, 0x94, 0x01, 0x8A, 0x06, 0x04, 0x8A, 0x4A,
    0x0F, 0x83, 0x43, 0x0C, 0x06, 0x0A, 0x04
};

const unsigned char lut_vcom1[] =
{
    0x03, 0x1D, 0x01, 0x01, 0x08, 0x23, 0x37, 0x37,
    0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

const unsigned char lut_red0[] =
{
    0x83, 0x5D, 0x01, 0x81, 0x48, 0x23, 0x77, 0x77,
    0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

const unsigned char lut_red1[] =
{
    0x03, 0x1D, 0x01, 0x01, 0x08, 0x23, 0x37, 0x37,
    0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

// Ecriture d'une donne.
void SpiTransfer(unsigned char data) {
    GPIO_setOutputLowOnPin(cs_port, cs_pin); // Slection du chip
    UCB0TXBUF = data;
    while (UCBUSY & UCB0STAT); // Attente fin de transmission avant de dselectionner le chip
    GPIO_setOutputHighOnPin(cs_port, cs_pin); // dselection du chip
}

/*
 *  N de port et pin pour les diffrents signaux.
 *  CS = Chip Select, DC = Data/Command RST = Reset, Busy = Busy
 */
void EpdHardwareInitSPIB()
{
    GPIO_setAsInputPin(busy_pin, busy_pin);
    GPIO_setOutputHighOnPin(dc_port, dc_pin);
    GPIO_setAsOutputPin(dc_port, dc_pin);
    GPIO_setOutputHighOnPin(reset_port, reset_pin);
    GPIO_setAsOutputPin(reset_port, reset_pin);
    GPIO_setAsOutputPin(MOSI_port, MOSI_pin);
    GPIO_setAsOutputPin(clk_port, clk_pin);

    // Configure SPI-4
    PORT_SEL0_MOSI |= BIT1 | BIT2 ;  // set 4-SPI pin as second function. SOMI is not used. BIT 0 = STE, BIT 1 = CLK, BIT2 = SIMO
    UCB0CTLW0 = UCSWRST;             // **Put state machine in reset**
                                     // 4-pin, 8-bit SPI master, Clock polarity high, MSB first, clock high (UCCKPL)
//    UCB0CTLW0 |= UCMST|UCSYNC|UCCKPL|UCMSB|UCMODE_2|UCSTEM|SMCLK;
    UCB0CTLW0 |= UCMST|UCSYNC|UCCKPL|UCMSB|UCMODE_2|UCSTEM|UCSSEL__SMCLK;

    UCB0BR0 = 0x02;                  // /2. Note, d'aprs la doc, fBRCLK/UCBRx. D'aprs certains exemples, fBitClock = fBRCLK/(UCBRx+1).
    UCB0BR0 = 0x00;                  // /2. Note, d'aprs la doc, fBRCLK/UCBRx. D'aprs certains exemples, fBitClock = fBRCLK/(UCBRx+1).
    UCB0BR1 = 0x00;

    UCB0IE &= ~(UCRXIE | UCTXIE);    // Disable USCI_B0 RX et TX interruption

    UCB0CTLW0 &= ~UCSWRST;           // Initialize USCI state machine**

    GPIO_setOutputHighOnPin(cs_port, cs_pin); // Normalement, PIN MISO en SPI
    GPIO_setAsOutputPin(cs_port, cs_pin);

}

/*
 * Initialisation du e-paper. EpdHardwareInitSPIB() doit avoir t pralablement appel
 */
void EpdInit() {
    EpdReset();
#ifdef AFFICHAGE_RAPIDE
    EpdSendCommand(POWER_SETTING);
    EpdSendData(0x07);
    EpdSendData(0x00);
    EpdSendData(0x08);
    EpdSendData(0x00);

    EpdSendCommand(BOOSTER_SOFT_START);
    EpdSendData(0x07);
    EpdSendData(0x07);
    EpdSendData(0x07);

    EpdSendCommand(POWER_ON);

    EpdWaitUntilIdle();

    EpdSendCommand(PANEL_SETTING);
    EpdSendData(0xcf);

    EpdSendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
    EpdSendData(0x17);          // valeur conseille

    EpdSendCommand(PLL_CONTROL);
    EpdSendData(0x39);          // Temprature >= 30C

    EpdSendCommand(TCON_RESOLUTION);
    EpdSendData(0xC8);          // valeur conseille (200 x 200)
    EpdSendData(0x00);          // valeur conseille (200 x 200)
    EpdSendData(0xC8);          // valeur conseille (200 x 200)

    EpdSendCommand(VCM_DC_SETTING_REGISTER);
    EpdSendData(0x0E);


#else
    EpdSendCommand(POWER_SETTING);
    EpdSendData(0x03);          // valeur conseille
    EpdSendData(0x00);          // valeur conseille
    EpdSendData(0x08);          // valeur conseille
    EpdSendData(0x08);          // valeur conseille

    EpdSendCommand(BOOSTER_SOFT_START);
    EpdSendData(0x07);          // valeur conseille
    EpdSendData(0x06);          // valeur conseille
    EpdSendData(0x05);          // valeur conseille

    EpdSendCommand(POWER_ON);

    EpdWaitUntilIdle();

    EpdSendCommand(PANEL_SETTING);
    EpdSendData(0xcf);

    EpdSendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
    EpdSendData(0x17);          // valeur conseille

    EpdSendCommand(PLL_CONTROL);
    EpdSendData(0x2A);          // Temprature < 30C (meilleurs rsultat en labo)

    EpdSendCommand(TCON_RESOLUTION);
    EpdSendData(0xC8);          // valeur conseille (200 x 200)
    EpdSendData(0x00);          // valeur conseille (200 x 200)
    EpdSendData(0xC8);          // valeur conseille (200 x 200)

    EpdSendCommand(VCM_DC_SETTING_REGISTER);
    EpdSendData(0x0C);          // valeur conseille


#endif

    EpdSetLutBw();
    EpdSetLutRed();

	CurrentByteBlack = 0;
	CurrentByteRed = 0;
	CurrentBank = BLACK;
    /* EPD hardware init end */
}

/**
 *  @brief: basic function for sending commands
 */
void EpdSendCommand(unsigned char command) {
    GPIO_setOutputLowOnPin(dc_port, dc_pin);
    SpiTransfer(command);
}

/**
 *  @brief: basic function for sending data
 */
void EpdSendData(unsigned char data) {
    GPIO_setOutputHighOnPin(dc_port, dc_pin);
    SpiTransfer(data);
}

/**
 *  @brief: Wait until the busy_pin goes HIGH
 */
void EpdWaitUntilIdle(void) {
    while(GPIO_getInputPinValue(busy_port, busy_pin) == GPIO_INPUT_PIN_LOW) {      //Low: busy, High: idle. La documentation Waveshare n'est pas cohrente. Active High dans une doc, Low dans l'autre.
        SleepMS(1);
    }

}

/**
 *  @brief: module reset.
 *          often used to awaken the module in deep sleep,
 *          see EpdSleep();
 */
void EpdReset(void) {
    GPIO_setOutputLowOnPin(reset_port, reset_pin);
    SleepMS(2);
    GPIO_setOutputHighOnPin(reset_port, reset_pin);
    SleepMS(2);
}

/**
 *  @brief: set the look-up tables
 */
void EpdSetLutBw(void) {
    unsigned int count;
    EpdSendCommand(0x20);         //g vcom
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_vcom0[count]);
    }
    EpdSendCommand(0x21);        //g ww --
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_w[count]);
    }
    EpdSendCommand(0x22);         //g bw r
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_b[count]);
    }
    EpdSendCommand(0x23);         //g wb w
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_g1[count]);
    }
    EpdSendCommand(0x24);         //g bb b
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_g2[count]);
    }
}

void EpdSetLutRed(void) {
    unsigned int count;
    EpdSendCommand(0x25);
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_vcom1[count]);
    }
    EpdSendCommand(0x26);
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_red0[count]);
    }
    EpdSendCommand(0x27);
    for(count = 0; count < 15; count = count+1) {
        EpdSendData(lut_red1[count]);
    }
}
/*
 * dmarre une session d'affichage : initialise les pointeurs mmoire  0
 * Rend True si pas d'erreur
 * Rend faux si on essaye de redmarrer du banc noir alors qu'on est sur le banc rouge
 * (il faut faire un EpdSessionClose => Refresh d'abord)
*/
bool EpdSessionStart(uint8_t Bank) {
	CurrentByteBlack = 0;
	CurrentByteRed = 0;
	SleepMS(1);
	if (Bank == RED) {
		CurrentBank = RED;
		EpdSendCommand(DATA_START_TRANSMISSION_2);
	} else {
		if (CurrentBank == RED) {return false;}
		CurrentBank = BLACK;
		EpdSendCommand(DATA_START_TRANSMISSION_1);
	}
	SleepMS(1);
	return true;
}
/*
* Rempli l'cran d'une couleur  partir de la position courante
* Color = UNCOLORED (blanc) ou COLORED (noir ou rouge).
*/
void EpdSessionFill(bool Color) {
	EpdSessionGoToLineRel(EPD_HEIGHT,Color);
}


/*
* Fin de session d'affichage. Refresh...
*/
void EpdSessionClose() {
    SleepMS(1);
	EpdSendCommand(DISPLAY_REFRESH);
	CurrentBank = BLACK;
	CurrentByteBlack = 0;
	CurrentByteRed = 0;
	EpdWaitUntilIdle();
}


/*
* Met l'effaceeur  blanc sur le banc BLACK et RED
*/

void EpdSessionClearDisplay() {
	EpdSessionStart(BLACK);
	EpdSessionFill(UNCOLORED);
	EpdSessionStart(RED);
	EpdSessionFill(UNCOLORED);
	EpdSessionClose();
}

/*
* Positionne le pointeur en dbut de la ligne L (0 = 1re ligne).
* ATTENTION : pas de retour en arrire possible. si L est infrieure ou gale
*  la ligne courante, la fonction ne fait pas d'action
* idem si L > taille cran
* La fonction rend true si elle a pu raliser l'action demande
* 
*/
bool EpdSessionGoToLineAbs(uint16_t L, bool Color)
{
	unsigned char temp;
	uint16_t *CurrentByte;
	uint16_t i,n;
	if (L >= EPD_HEIGHT) {return false;}
	if (CurrentBank == RED) {
		CurrentByte = &CurrentByteRed;
	    L = EPD_LINE_BYTE_RED * L;
	} else {
		CurrentByte = &CurrentByteBlack;
	    L = EPD_LINE_BYTE_BW * L;
	}
	if (L < *CurrentByte) {return false;}
	n = L - *CurrentByte;
	if (Color) {temp = 0x00;} else {temp = 0xFF;}
	for (i = n; i > 0; i = i-1) {
		EpdSendData(temp);
		*CurrentByte += 1;
	}
	return true;
}

/*
* Positionne le pointeur L lignes aprs la ligne courante (0 = 1re ligne).
* ATTENTION : pas de retour en arrire possible.
* Note : si L=0, la fonction termine la ligne courante
* Color = True pour Black ou Red = COLORED, false pour White = UNCOLORED
* 
*/

void EpdSessionGoToLineRel(uint16_t L, bool Color)
{
	unsigned char temp;
	uint16_t i,n;
    uint16_t *CurrentByte;

    if (Color) {temp = 0x00;} else {temp = 0xFF;}
	if (CurrentBank == RED) {
	    n = ((L+1)*EPD_LINE_BYTE_RED) - (CurrentByteRed % (EPD_LINE_BYTE_RED));
	    if (n > EPD_MEM_BYTE_RED) {n = EPD_MEM_BYTE_RED;}
        CurrentByte = &CurrentByteRed;
	} else {
	    n = ((L+1)*EPD_LINE_BYTE_BW) - (CurrentByteBlack % (EPD_LINE_BYTE_BW));
	    if (n > EPD_MEM_BYTE_BW) {n = EPD_MEM_BYTE_BW;}
        CurrentByte = &CurrentByteBlack;
	}
    for (i = n; i > 0; i = i-1) {
        EpdSendData(temp);
        *CurrentByte += 1;
	}
}

/**
 *  Ecriture de la string text.
 * Par dfaut, criture caractres Noir sur blanc (banc noir et blanc) ou rouge sur blanc (banc rouge)
 * InvertedChar = true : inversion couleur fond et couleur caractres
 * InvertedFill = true : si banc noir et blanc, remplisage noir. Si banc rouge, remplissage rouge. Sinon, remplissage blanc
 * nrFont = n de fonte
 * nrSpace = nombre de caractres "espace" avant l'criture du texte
 */
void EpdSessionWriteString(uint8_t nrSpace, const char * text, uint8_t nrFont, bool InvertedChar, bool InvertedFill) {
    int i, k, l, n;
    uint16_t char_offset;
	unsigned char buffer[EPD_WIDTH]; 
	uint16_t ibuffer;
	uint16_t itext;
	uint16_t ispace;
	unsigned char temp;
	uint16_t max;
	unsigned char mask;
	unsigned char maskDefault;
	unsigned char shift;
	uint16_t *CurrentByte;
	sFONT *CurrentFont;					// Fonte courante
	

	switch (nrFont)
	{
		case 20: CurrentFont = &Font20;	break;
		case 15: CurrentFont = &Font15; break;
		case 4:  CurrentFont = &Font4line;	break;
		default: CurrentFont = &Font20;	break;
	}
	if (CurrentBank == RED) {
		CurrentByte = &CurrentByteRed;
		maskDefault = 0x80;
		shift = 1;
	} else {
		CurrentByte = &CurrentByteBlack;
		maskDefault = 0xC0;
		shift = 2;
	}
	for (k = 0; k < CurrentFont->Height; k++) { // Pour toute la hauteur des caractres
		if (CurrentBank == RED) {
			max = CurrentByteRed+EPD_LINE_BYTE_RED-(CurrentByteRed % (EPD_LINE_BYTE_RED));		// 1 bit par pixel sur fond rouge
		} else {
			max = CurrentByteBlack+EPD_LINE_BYTE_BW-(CurrentByteBlack % (EPD_LINE_BYTE_BW)); 	// 2 bits par pixel sur fond noir et blanc
		}
		ibuffer = 0;
		itext = 0;
		ispace = nrSpace;
		while ((text[itext] != 0) || (ispace)) {  // Pour tous les caractres
			if (ispace) {
				char_offset = (' ' - CurrentFont->Base) * CurrentFont->Height * (CurrentFont->Width / 8 + (CurrentFont->Width % 8 ? 1 : 0));
			} else {
				char_offset = (text[itext] - CurrentFont->Base) * CurrentFont->Height * (CurrentFont->Width / 8 + (CurrentFont->Width % 8 ? 1 : 0));
			}
			l = CurrentFont->Width;
			for (i = 0; i <= ((CurrentFont->Width-1)/8); i++) {
			    temp = CurrentFont->table[char_offset+i+(k*(1+((CurrentFont->Width-1) / 8)))];
				n = 0;
				while ((n < 8) && (l > 0)) {
					buffer[ibuffer] = temp & 0x80;
					temp = (temp << 1);
					ibuffer = ibuffer+1;
					if (ibuffer >= EPD_WIDTH) {break;}
					n = n+1;
					l = l-1;
				}
			}
			if (ispace) {ispace = ispace-1;} else {itext +=1;}
		}
		
		temp = 0xFF;
		mask = maskDefault;
		for (i = 0; i < ibuffer;i++) {
			if (buffer[i]) {temp &= ~mask;}
			mask = (mask >> shift);
			if (!mask) {
				if (*CurrentByte < max) {
					if (InvertedChar) {temp = ~temp;}
					EpdSendData(temp);
					*CurrentByte += 1;
				}
				temp = 0xFF;
				mask = maskDefault;
			}
		}
		if (mask != maskDefault) {
			if (*CurrentByte < max) {
				if (InvertedChar) {temp = ~temp;}
				EpdSendData(temp);
				*CurrentByte += 1;
			}
		}
		EpdSessionGoToLineRel(0,InvertedFill); 
	}
}


/**
 *  @brief: After this command is transmitted, the chip would enter the
 *          deep-sleep mode to save power.
 *          The deep sleep mode would return to standby by hardware reset.
 *          The only one parameter is a check code, the command would be
 *          executed if check code = 0xA5.
 *          You can use EpdInit() to awaken
 */
void EpdSleep() {
    EpdSendCommand(VCOM_AND_DATA_INTERVAL_SETTING);
    EpdSendData(0x17);
    EpdSendCommand(VCM_DC_SETTING_REGISTER);         //to solve Vcom drop
    EpdSendData(0x00);
    EpdSendCommand(POWER_SETTING);         //power setting
    EpdSendData(0x02);        //gate switch to external
    EpdSendData(0x00);
    EpdSendData(0x00);
    EpdSendData(0x00);
    EpdWaitUntilIdle();
    EpdSendCommand(POWER_OFF);         //power off
}
/*
* Rend la largeur (Width) et la hauteur (Height) en pixels de la fonte nrFont
* Rend 0 si la fonte n'existe pas
*/
int  EpdSessionGetFontWidth(uint8_t nrFont)
{
	switch (nrFont)
	{
		case 20: return Font20.Width;
        case 15: return Font15.Width;
		case 4:  return Font4line.Width;
		default: return 0;
	}
}
	
int  EpdSessionGetFontHeight(uint8_t nrFont)
{
	switch (nrFont)
	{
		case 20: return Font20.Height;
        case 15: return Font15.Height;
		case 4:  return Font4line.Height;
		default: return 0;
	}
}	



/* END OF FILE */


