/*
 * File:        common.h
 * Purpose:     Common defines
 * Copyright:   (C) Pascal Chour
 * Licence:     GPL
 */
#ifndef __INCLUDE_COMMON_H
#define __INCLUDE_COMMON_H

#include <msp430fr4133.h>
#include "stdint.h"
#include "stdbool.h"

/* Define pour programme Geiger - Pascal Chour */

//#define TEST_LCD // gnration de code pour la version en dveloppement => Tests LCD

/* Ports et PIN utiliss */

#define PORT_TX GPIO_PORT_P1 // TX processeur (out)
#define PIN_TX GPIO_PIN0

#define PORT_RX GPIO_PORT_P1 // RX Processeur (in)
#define PIN_RX GPIO_PIN1

#define PORT_BAT_VOLT GPIO_PORT_P1 // Lecture tension batterie (in)
#define PIN_BAT_VOLT GPIO_PIN3

#define PORT_RST GPIO_PORT_P1 // RST e-paper (out)
#define PIN_RST GPIO_PIN4

#define PORT_BUSY GPIO_PORT_P1 // BUSY e-paper (in)
#define PIN_BUSY GPIO_PIN5

#define PORT_AL_BUZ GPIO_PORT_P1 // Alarme buzzer (out)
#define PIN_AL_BUZ GPIO_PIN6

#define PORT_POUSSOIR GPIO_PORT_P1 // Poussoir (in)
#define PIN_POUSSOIR GPIO_PIN7
#define PxIFG_POUSSOIR P1IFG
#define PxIV_POUSSOIR P1IV_P1IFG7

#define PORT_ROUE_B GPIO_PORT_P2 // Roue codeuse B (in)
#define PIN_ROUE_B GPIO_PIN5
#define PxIV_ROUEB P2IV_P2IFG5

#define PORT_ROUE_A GPIO_PORT_P2 // Roue codeuse A (in)
#define PIN_ROUE_A GPIO_PIN7
#define PxIV_ROUEA P2IV_P2IFG7

#define PxIFG_ROUE_AB P2IFG // Commun  A et B

#define PORT_DC GPIO_PORT_P5 // DC e-paper (out)
#define PIN_DC GPIO_PIN0

#define PORT_CLK GPIO_PORT_P5 // CLK e-paper (out)
#define PIN_CLK GPIO_PIN1

#define PORT_MOSI GPIO_PORT_P5 // MOSI (out)
#define PIN_MOSI GPIO_PIN2
#define PORT_SEL0_MOSI P5SEL0

#define PORT_CS GPIO_PORT_P5 // CS (Chip Select)
#define PIN_CS GPIO_PIN3

#define PORT_AL_LED GPIO_PORT_P8 // AL_LED (out)
#define PIN_AL_LED GPIO_PIN0

#define PORT_SLEEP GPIO_PORT_P8 // Sleep (in)
#define PIN_SLEEP GPIO_PIN1

#define PORT_DETECT1 GPIO_PORT_P8 // DETECT1 (in)
#define PIN_DETECT1 GPIO_PIN2

#define PORT_BAT_EN GPIO_PORT_P8 // BATen (out)
#define PIN_BAT_EN GPIO_PIN3



/* variables d'tat du systme */

#define periode_affichage (5)           // Periode pour rafraichissement LCD et calculs en secondes. Typ : 5 secondes
#define periode_par_minute (60/periode_affichage)    // Nombre de periodes par minute.
#define nb_minutes_par_24h  (1440)      // nb de minutes en 24H
#define alarme_defaut (50)              // valeur de seuil pour alarme par dfaut
#define NbMoyHeures (500)               // Nombre d'heures mmorises dans le buffer circulaire

#define INACTIVITE_MAX (20)             // pour sortie du mode dialogue si inactivit (en secondes)
#define ALARME_MAX     (10)             // dure de l'alarme en secondes

/* seuil Tensions batteries. Tension centivolts */

#define V_PROC_SEG_1 (300)              // Tension processeur. Tension faible
#define V_PROC_SEG_2 (315)              // Tension processeur. Tension un peu faible.
#define V_PROC_SEG_3 (330)              // Tension processeur. tension nominale

#define V_BAT_SEG_1 (345)               // Batterie 1 segment
#define V_BAT_SEG_2 (355)               // Batterie 2 segment
#define V_BAT_SEG_3 (365)               // Batterie 3 segment
#define BUF_MAX (30)                    // taille max d'un buffer. A priori, le max est ID=XXXX,XXXX,XXXX,XXXX
#define MaxPileES (4)                   // Taille max pile pour commandes entres sorties

typedef enum {MENU_REPOS, MENU_PAPER, MENU_LED_change, MENU_LED, MENU_BUZ_change, MENU_BUZ, MENU_ALARME_change, MENU_ALARME, MENU_EXIT} T_ETAT_MENU ;


typedef struct
{
  bool a_traiter;       // vrai si quelque chose  traiter
  uint8_t  index;       // index courant du buffer
  char buf[BUF_MAX];    // buffer  traiter
} tESbuf;
  
typedef struct // mmorisation des moyennes par heures
{
    uint16_t Buf[NbMoyHeures]; // contient les dernires NbMoyHeures moyennes par heures
    int16_t Index; // Index courant pour stockage dans MoyHeures. Doit tre initialis  -1 au dmarrage
    bool Plein; // Indique si le buffer circulaire est plein.
    int16_t IndexES; // Index courant de la valeur  mettre dans MoyHeures suite  une commande Download
} TMoyHeures;

// PileES et ses fonctions associes (PUSH, POP...) permettent de mmoriser les
// actions  traiter au niveau E/S. On limite  MaxPilesES+1 le nombre d'actions.
// CmdDL1 => mettre le premier DL=xxxx ou NODATA
// CmdDL2 => mettre les DL=xxxx suivants jusqu' END
// CmdDL3 => mettre DL=END
// CmdCM => mettre un CM=xxxx
// CmdCF => mettre CF=xxxx
// cmdVP => mettre VP=... (N) de version, type compteur, n de srie)
// CmdPC => mettre paramtres compteurs
enum {CmdDL1, CmdDL2, CmdDL3, CmdVide, CmdCM, CmdCF, CmdVP, CmdPC, CmdGD};

typedef struct {
    int16_t Index;
    char Commande[MaxPileES+1];
} TPileES;

// Variables globales externes
volatile extern uint32_t countMoyenneHeure[25];         // nb d'heures +1 par jours (main.c)
volatile extern TMoyHeures MoyHeures;
volatile extern uint16_t NbMinutes24h;                  // nb de minutes coules depuis le dbut du comptage limit  24h00 (main.c)
volatile extern uint16_t NbJours;                       // Nb de jours depuis que la mise en marche du compteur
volatile extern uint16_t CountMax;                      // Valeur max courante. 0 = pas initialise
volatile extern int8_t Sens;                            // Sens de rotation et nombre d'impulsion
volatile extern bool Poussoir;                          // Poussoir
volatile extern T_ETAT_MENU ETAT_MENU;                  // Etat du menu
volatile extern uint8_t CompteInactivite;               // compteur d'inactivit pour sortir du menu (en secondes)
volatile extern bool MemoireAlarme;
volatile extern bool BasculeActivite;                   // pour faire clignoter l'indicateur d'activit
volatile extern uint8_t CompteAlarme;
volatile extern uint32_t CountParMinute;                // Pour compter le nombre d'impulsions par minutes. Modifiable par IT.
extern uint16_t Vbat;                                   // dernire tension batterie mesure
extern bool AlarmeLed;                                  // vrai si on fait clignoter la LED si alarme */
extern bool AlarmeBuz;                                  // vrai si on active le buzzer si alarme */
extern volatile bool ValideAlarme;                      // Indique qu'on doit valider l'alarme
extern uint16_t SeuilAlarme;                            // Seuil  partir duquel on gnre une alarme (main.c)
extern volatile uint8_t ETAT_ALARME;

extern const char T_Invite[];
extern const char T_ON[];
extern const char T_OFF[];
extern const char T_LED[];
extern const char T_BUZ[];
extern const char T_AL[];

#endif // __INCLUDE_COMMON_H 
