/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include "main.h"
#include "common.h"
#include "hal_LCD.h"
#include "PC_Epd_154_low_memory.h"
#include "Affepaper.h"
#include "menu.h"
#include "utilitaires.h"


/* Variables d'tat du systme (globales) */

volatile uint16_t countMoyenneMin[periode_par_minute]; // pour mmorisation de moyennes sur 1 mn glissante. Modifiable par IT.
volatile uint32_t countMoyenneHeure[25];         // nb d'heures +1 par jours. Modifiable par IT.
volatile TMoyHeures MoyHeures;
volatile uint16_t NbJours = 1;                   // Pour mmorisation du nombre de jours de fonctionnement du compteur depuis son dmarrage. Modifiable par IT.
volatile uint16_t NbMinutes24h = 0;              // nb de minutes coules depuis le dbut du comptage limit  24h00. Modifiable par IT.
volatile uint8_t NbMinutes = 0;                  // nb de minutes pour le passage des heures. Modifiable par IT.
volatile uint8_t CptPeriode = 0;                 // pour comptage priode affichage et calcul. Evite une opration modulo. Modifiable par IT.
volatile uint8_t NbTiPmn = 0;                    // index courant dans tableau comptage par minutes. Modifiable par IT.
volatile uint8_t NbTiPh = 0;                     // index courant dans tableau comptage par heures. Modifiable par IT.
volatile uint8_t CompteAlarme = 0;               // Pour compter la signalisation de l'alarme courante (en secondes). Modifiable par IT.
volatile uint8_t CompteInactivite = 0;           // compteur d'inactivit pour sortir du menu (en secondes). Modifiable par IT.
volatile uint32_t CountParMinute = 0;            // Pour compter le nombre d'impulsions par minutes. Modifiable par IT.
volatile uint16_t CountMax = 0;                  // Valeur max courante. 0 = pas initialise. Modifiable par IT.
volatile uint16_t CountLast = 0;                 // dernire valeur du compteur d'impulsion. Modifiable par IT.
volatile T_ETAT_MENU ETAT_MENU = MENU_REPOS;     // Le menu est au repos.  Modifiable par IT.

volatile bool BasculeActivite = false;           // pour faire clignoter l'indicateur d'activit
volatile bool MemoireAlarme = false;             // Pour indiquer qu'on a une alarme  valider
volatile bool ForceAffPaper = false;             // Pour forcer l'affichage e-paper au bout de 24h
volatile bool ValideAlarme = false;              // Pour forcer l'affichage et valider l'alarme





/*
 * main.c
 */
int main(void) {
uint16_t i;
/*
 * dans cette section, on ne doit pas avoir besoin des interruptions pour fonctionner !
 */

    // Au dmarrage, les interruptions sont supposes masques (bit GIE de SR  0).
    // Stop Watchdog timer
     WDT_A_hold(__MSP430_BASEADDRESS_WDT_A__);     // Stop WDT
/*
 * Dans cette section, on ne peut appeler que des routines qui n'utilisent pas de timers
 * ou d'autres interruptions.
 */
    InitGenerale(); // doit tre appel en premier. Initialisation par dfaut des GPIO, horloges, certaines variables, etc.

    Init_LCD(); // initialisation de l'afficheur LCD

    // principe de comptage : un tableau comportant "periode_par_minute" positions
    // est utilis pour calculer une moyenne glissante. Ex : si une priode de comptage est
    // 10s, alors, il y a 6 priodes de comptage par minute. Ce tableau est initialis
    //  0 au dmarrage.
    // Il faut attendre une minute pour que la moyenne de comptage soit
    // correcte.
    for (i = 0; i < periode_par_minute;i++) {countMoyenneMin[i] = 0;}

    // on initialise les tableaux pour comptage par heure sur 24H
    for (i = 0; i < 25; i++) {countMoyenneHeure[i] = 0;}

    // Buffer des heures mmorises
    for (i = 0; i < NbMoyHeures; i++) {MoyHeures.Buf[i] = 0;}
    MoyHeures.Index = -1;
    MoyHeures.Plein = false;
    MoyHeures.IndexES = -1;

    __enable_interrupt(); // Dmasquage des interruptions dans le Status Register (bit GIE de SR)

/*
 * A partir de cettes section, toutes les IT utiles sont actives. On peut donc utiliser l'afficheur LCD, la roue codeuse ou le poussoir, etc.
 * Le RTC tant actif, on voit le symbole d'activit clignoter sur l'afficheur LCD
 */
    displayScrollText(T_Invite); // Affichage d'un petit texte d'introduction
    LCD_displayLevel(0, true); // Affichage des barres droites et gauche de l'indicateur de niveau.

#ifdef TEST_LCD             // Squence de test du LCD
    for (i=0; i <= 6; i++) {
        LCD_displayLevel(i,true);
        Delay250(1);
    }
    for (i=6; i != 0; i--) {
        LCD_displayLevel(i-1,true);
        Delay250(1);
    }
    LCD_displayLevel(0, false);
    LCD_displayTmr(true);
    Delay250(1);
    LCD_displayTmr(false);
    LCD_displayAnt(true);
    Delay250(1);
    LCD_displayAnt(false);
    LCD_displayAlarm(true);
    Delay250(1);
    LCD_displayAlarm(false);
    LCD_displayHeart(true);
    Delay250(1);
    LCD_displayHeart(false);
    LCD_Clear();            // effacement de l'afficheur
#endif

    EpdHardwareInitSPIB(); // Initialisation ports et contrleur SPI pour le module e-paper
    InitialiseES();
    AffDonnees(false, true); // Premier affichage sur le e-paper. On force les affichages (batterie, MIN, MAX, MOY...)

    while(1) {
        MENU_automate(); // On vrifie s'il y a quelque chose  faire au niveau du dialogue homme-machine
        InitialiseES(); // Teste si on doit initialiser ou clore les E/S
        // Doit on forcer l'affichage (par ex. Toutes les 24h) ou valider une alarme ou vrifier s'il ne faut pas modifier l'affichage
        // car une alarme est en cours ou les trois !
        if ((ForceAffPaper) || (ValideAlarme) || (CompteAlarme)) {
            AffDonnees(ValideAlarme, ForceAffPaper);
            ValideAlarme = false;
            ForceAffPaper = false;
        }
        LPM3; // On s'endord
    }
}


/*
 * RTC Interrupt Service Routine
 * Rveil toutes les secondes. Effectue la mesure du comptage et l'affichage sur l'cran LCD.
 * Met  jour l'indicateur d'alerte si besoin. Fait clignoter l'indicateur d'activit
 */
#pragma vector = RTC_VECTOR
__interrupt void RTC_ISR(void)
{
    uint16_t CountCurrent;
    uint16_t i;
    uint32_t Bid;
    LPM3_EXIT;
    switch(__even_in_range(RTCIV, RTCIV_RTCIF))
    {
        case RTCIV_NONE : break;
        case RTCIV_RTCIF:
            switch (CptPeriode)
            {
                case 0 :   // Toutes les "periodes_affichage"
                    countMoyenneMin[NbTiPmn] = 0;
                    CountCurrent = TA1R; /* lecture du compteur courant */
                    if (CountCurrent < CountLast) { /* Petit traitement si le compteur a dbord. On suppose qu'il ne peut y avoir plus de FFFF impulsions entre deux calculs */
                        countMoyenneMin[NbTiPmn] = (0xFFFF-CountLast)+CountCurrent;
                    } else {
                        countMoyenneMin[NbTiPmn] = CountCurrent-CountLast;
                    }
                    CountLast = CountCurrent; // On conserve la valeur pour le coup d'aprs
                    // Nb de comptage par minutes. Il faut attendre une minute pour que le comptage soit valide.
                    CountParMinute = 0;
                    for (i = 0; i < periode_par_minute; i++) {
                        CountParMinute = CountParMinute+countMoyenneMin[i];
                    }

                    // Traitement du max et de l'alarme
                    if (CountParMinute > CountMax) {CountMax = CountParMinute;}
                    if (CountParMinute > SeuilAlarme) {
                        if (CompteAlarme == 0) {
                            MemoireAlarme = true;           // Pour signaler l'alarme sur l'afficheur LCD
                            ETAT_ALARME = E_ALARME0;       // Initialisation de l'automate de signalement d'alarme
                            if (!(TA0CTL & (MC0 | MC1))) {  // Si le timer est  arrt
                                TA0CCTL0 = CCIE;            // Autorise interruption sur CCIFG : Interruption si le compteur atteind une certaine valeur.
                                TA0CCR0 = PERIODE_A0;       // Valeur  comparer pour interruption. Correspond environ  1ms
                                TA0R = 0;                   // Compteur dmarre  0
                                TA0CTL = TASSEL_1 + MC_2;   // TASSEL_1 => utilisation de ACLK (32768Hz), MC_1 => compare mode jusqu' TA0CCR0
                            }
                        }
                        CompteAlarme = ALARME_MAX;           // Pour commencer  compter la dure de signalement de l'alarme LED ou Buzzer
                    }
                    NbTiPmn = NbTiPmn+1;

                    if (NbTiPmn >= periode_par_minute) {// Traitement toutes les minutes
                        NbTiPmn = 0;

                        // On suppose qu'il ne peut pas y avoir de dbordement car compteur 32 bits ! Pour mmoire, 1000 coups par seconde * 60 * 60 = 3600000.
                        countMoyenneHeure[NbTiPh] += CountParMinute;
                        if (CountParMinute >= 0xFFFF) {CountParMinute = 0xFFFF;}
                        PileESPUSH(CmdCM);

                        if (NbMinutes24h <= nb_minutes_par_24h) {NbMinutes24h = NbMinutes24h+1;} // utilis pour le calcul de la moyenne

                        NbMinutes = NbMinutes+1; // Et par ailleurs, on a un compteur de minutes pour faire passer les heures
                        // On force l'affichage complet du e-paper toutes les 00H05MN ce qui permet galement
                        // de raffraichir l'afficheur 5mn aprs le dmarrage
                        if ((NbMinutes == 5) && (NbTiPh == 0)) {ForceAffPaper = true;}
                        if (NbMinutes > 60) {
                          NbMinutes = 0;
                          MoyHeures.Index = MoyHeures.Index+1;
                          if (MoyHeures.Index >= NbMoyHeures) {
                              MoyHeures.Index = 0;
                              MoyHeures.Plein = true; // le buffer circulaire est plein
                          }
                          Bid = countMoyenneHeure[NbTiPh]/60;
                          if (Bid > 0xFFFF) {
                              MoyHeures.Buf[MoyHeures.Index] = 0xFFFF;
                          } else {
                              MoyHeures.Buf[MoyHeures.Index] = Bid;
                          } // moyenne des CPM par heures
                          NbTiPh = NbTiPh+1;
                          if (NbTiPh > 24) {
                              NbTiPh = 0;
                              NbJours = NbJours+1;
                          }
                          countMoyenneHeure[NbTiPh] = 0;
                        }
                    }
                    // traitement de l'affichage
                    // si pas d'affichage du menu, alors, on affiche le comptage
                    if (ETAT_MENU == MENU_REPOS)  {LCD_writeIntRight(CountParMinute);}

                    /* affichage bargraph chelle log */
                    if (CountParMinute == 0) {
                        LCD_displayLevel(0,true);
                    } else {
                      if (CountParMinute < 20) {
                          LCD_displayLevel(1,true);
                      } else {
                        if (CountParMinute < 40) {
                            LCD_displayLevel(2,true);
                        } else {
                          if (CountParMinute < 80) {
                              LCD_displayLevel(3,true);
                          } else {
                            if (CountParMinute < 160) {
                                LCD_displayLevel(4,true);
                            } else {
                              if (CountParMinute < 320) {
                                  LCD_displayLevel(5,true);
                              } else {
                                  LCD_displayLevel(6,true);
                              }
                            }
                          }
                        }
                      }
                    }
                    LCD_displayAlarm(MemoireAlarme); // affichage du symbole signalant une alarme

                default :
                    // A faire chaque seconde, y compris lorsqu'on est occup  grer le e-paper (bloquant et trs long). Si pas de e-paper ou de processus lents de ce type
                    // il faut passer cette partie dans le main()
                    BasculeActivite = !BasculeActivite;
                    LCD_displayHeart(BasculeActivite);
                    if (ETAT_MENU != MENU_REPOS) {
                        if (CompteInactivite > 0) {CompteInactivite--;} else {ETAT_MENU = MENU_REPOS; Sens = 0; Poussoir = 0; LCD_ClearText();}
                    }
                    if (CompteAlarme > 0) {CompteAlarme--;}
                break;
            }
            if (CptPeriode >= periode_affichage) {CptPeriode = 0;} else {CptPeriode = CptPeriode+1;}   // mise   jour du compteur de seconde. Remis  zro toutes les priodes de temps
        break;
    }
}


