/**
 * Fichier : utilitaires.h
 * Auteur : Pascal Chour - www.pascalchour.fr
 * Date : 15 septembre 2019
 * (c) Pascal Chour 
 *-------------------------------------------------------------------------- 
 */

#ifndef UTILITAIRES_H
#define UTILITAIRES_H
#include <msp430fr4133.h>
#include "stdint.h"
#include "driverlib/MSP430FR2xx_4xx/driverlib.h"


/*
Timer en "mode normal". La mise  jour du comptage a lieu toutes les TIMER_NORMAL secondes.
Lorsqu'il y a une alarme ou que le compteur est aliment par USB, la mise
 jour se fait toutes les secondes (watchdog).
De mme, les contrles accessoires (vrification batterie, bouton poussoir, etc. ne se font que
toutes les TIMER_NORMAL secondes.
*/
#define FREQ_FONC (4000)            // Frquence de fonctionnement / 1000
/*
 * TA0 est utilis pour les anti-rebonds et la mise en sommeil en attente d'vnements
 * La priode est de 10ms environ.
 */
#define PERIODE_A0 (33)            // Priode du compteur A0 pour environ 1ms (par rapport  ACLK = 32768Hz)
/*
 * Cette valeur est le rapport entre la tension mesure par l'ADC et la tension de la batterie multipli par 100
 * A mesurer au voltmtre sur P1.3. A amliorer...
 */
#define FACTOR_BATTERIE (339)
/*
 * Dure d'un signal d'alarme (identique pour LED et buzzer). Pour l'alarme LED, on clignote toutes les PERIODE_LED
 * Pour le Buzzer, on met 2 coups bref puis on attend, puis on recommence
 */
#define DUREE_SIGNAL_ALARME (50)    // Dure d'un signal en ms
#define PERIODE_LED_BUZ (1000)      // Dlai entre deux flash ou double bips en ms
#define DELAI_BUZ (100)             // Pour faire 2 bips proches en ms
// Etat automate pour signalement alarme
// E_ALARME0  Etat d'entre. Initialisation variable d'tat
// E_ALARME1  On active la LED et/ou le buzzer, on initialise le timer pour la dure d'activation
// E_ALARME2  On dsactive la LED et/ou le buzzer. On initialise le timer pour le buzzer pour la dure de dsactivation
// E_ALARME3  On active le buzzer. On initialise le timer pour la dure d'activation.
// E_ALARME4  on dsactive le buzzer. On initialise le timer pour le dlai entre deux signaux -> E_ALARME1

enum {E_ALARME0,E_ALARME1,E_ALARME2,E_ALARME3,E_ALARME4};
/*
 * Tempo en ms pour l'antirebond.
 */
#define TEMPO_DEBOUNCE (75)              // Dure de l'anti-rebond en ms

uint16_t max16(uint16_t a,uint16_t b);
void Uint16Ascii(uint16_t Valeur, char T[]);
void SleepMS(uint16_t Nb);
void InitGenerale(void);
void InitialiseES();
void PileESPUSH(uint8_t Commande);

#endif /* UTILITAIRES_H */

/* END OF FILE */
